# coding:utf8
from __future__ import annotations
import hashlib
import json
import os.path
import pickle
import sys
import uuid
from datetime import datetime
import Levenshtein
import numpy as np
import pandas as pd
from pandas._typing import DtypeArg
from tqdm import tqdm


def generate_uuid():
    try:
        uuid4 = uuid.uuid4()
        return str(uuid4).upper().replace('-', '')
    except Exception as e:
        print('generate_uuid error:', e)
        return None


def generate_uuid_md5(data, start='C1'):
    try:
        md5 = hashlib.md5(data.encode(encoding='utf8')).hexdigest().upper()[2:]
        return start + md5[0:6] + '-' + md5[6:10] + '-' + md5[10:14] + '-' + md5[14:18] + '-' + md5[18:]
    except Exception as e:
        print('get_md5 error:', e)
        return None


def get_time():
    # 2022-03-03 16:12:30
    return datetime.now().strftime("%Y-%m-%d %X")


def read_excel(path, is_csv=False, sheet_name: str | int | list | None = 0, header: int | None = 0, is_value=True, is_replace=True,
               is_ffill=False, dtype: DtypeArg | None = None):
    """
    读取Excel
    :param path:
    :param sheet_name:
    :param header: 传None表示第一行是数据，不传（默认为0）表示第一行是列名
    :param is_value:
    :param is_replace:
    :param is_ffill: 单元格为空值（NaN，非空字符串）时，填充上一行的值，适用于合并单元格的情况。
    :return:
    """
    if not isinstance(sheet_name, list):
        if is_csv:
            df = pd.read_csv(path, dtype=dtype)
        else:
            df = pd.read_excel(path, sheet_name=sheet_name, header=header, dtype=dtype)
        if is_ffill:
            df = df.ffill()

        # nan替换为空字符串
        if is_replace:
            df = df.replace(np.nan, '', regex=True)

        if is_value:
            return df.values
        else:
            return df
    else:
        df_dic = {}
        for key, df in pd.read_excel(path, sheet_name=sheet_name, header=header, dtype=dtype).items():
            if is_ffill:
                df = df.ffill()

            # nan替换为空字符串
            if is_replace:
                df = df.replace(np.nan, '', regex=True)

            if is_value:
                df_dic[key] = df.values
            else:
                df_dic[key] = df

        return df_dic


def write_excel(path, res_list=[], columns=[], df=None):
    if df is None:
        df = pd.DataFrame(res_list, columns=columns)
    # 加engine、encoding参数是防止“openpyxl.utils.exceptions.IllegalCharacterError”
    # df.to_excel(path, engine='xlsxwriter', index=False, encoding='utf-8')
    df.to_excel(path, engine='xlsxwriter', index=False)


def df_concat(df1, df2, ignore_index=True):
    """

    :param df1:
    :param df2:
    :param ignore_index: True：重新排序，False：保留原有两个df的index
    :return:
    """
    return pd.concat([df1, df2], ignore_index=ignore_index)


def df_to_sql(path_df, table, engine, is_df=False):
    """
    df import to sql
    :param path_df:
    :param table: sql table
    :param engine: sqlalchemy engine
    :param is_df: True为DataFrame，False为Excel路径
    :return:
    """
    # pandas批量导入mysql
    if is_df:
        df = path_df
    else:
        df = read_excel(path_df, is_value=False)
    df.to_sql(table, engine, if_exists='append', index=False)


def get_extension(path):
    """
    获取扩展名（小写）
    :param path:
    :return:
    """
    if not os.path.exists(path):
        raise ValueError('file not exists!')
    return os.path.splitext(path)[-1].lower()


def read_txt(path):
    data_list = []
    with open(path, encoding='utf8') as f:
        for line in f:
            data_list.append(line.strip())
    return data_list


def dump_json(obj, path):
    json.dump(obj, open(path, 'w', encoding='utf8'), ensure_ascii=False)


def load_json(path):
    return json.load(open(path, encoding='utf8'))


def save_pkl(data, path):
    with open(path, "wb") as f:
        pickle.dump(data, f)


def load_pkl(path):
    with open(path, "rb") as f:
        data = pickle.load(f)
    return data


def generate_dict(list1, list2=[]):
    try:
        if not isinstance(list1, list):
            list1 = list(list1)
        if list2:
            if not isinstance(list2, list):
                list2 = list(list2)
            return dict(zip(list1, list2))
        else:
            return dict(zip(list1, range(len(list1))))
    except Exception as e:
        print(f"generate_dict error:{e}")
        return {}


def get_max_check(check_base_list, check_refer_list, max_ratio=0.5, is_ret_lr=False):
    result_list = []
    for check_base in check_base_list:
        max_lr = -1
        max_check = ''
        for check_refer in check_refer_list:
            rcc_lr = Levenshtein.ratio(check_base, check_refer)
            if rcc_lr < max_ratio:
                continue
            if rcc_lr > max_lr:
                max_lr = rcc_lr
                max_check = check_refer
        if max_lr >= max_ratio:
            if is_ret_lr:
                result_list.append([check_base, max_check, max_lr])
            else:
                result_list.append([check_base, max_check])
        else:
            if is_ret_lr:
                result_list.append([check_base, '', max_lr])
            else:
                result_list.append([check_base, ''])
    return result_list


def except_info(e=None):
    t, v, tb = sys.exc_info()
    # f'当前类名称：{self.__class__.__name__}'
    # f"当前方法名：{sys._getframe().f_code.co_name}"
    if e is None:
        return {'lineno': tb.tb_lineno, 'error': '%s(\'%s\')' % (v.__class__.__name__, str(v))}
    else:
        return {'file': e.__traceback__.tb_frame.f_globals["__file__"], 'line': tb.tb_lineno,
                'error': '%s(\'%s\')' % (v.__class__.__name__, str(v))}



# coding: utf8
# https://blog.csdn.net/qq_45797116/article/details/123609056
from multiprocessing.dummy import Pool
from multiprocessing import cpu_count

# 这种方式可能会把CPU占满
# def multi_pro(thread_num, func, param_list, is_ret=False):
#     """
#     多线程
#
#     Parameters
#     ----------
#     :param thread_num: 线程数
#     :param func: 循环数
#     :param param_list: 参数
#     :param is_ret: 是否返回最终结果
#
#     Examples
#     --------
#     >>> def func(num):
#     >>>     # 平方函数
#     >>>     return num * num
#
#     >>> param_list = [0, 1, 2]
#     >>> res = multi_pro(3, func, param_list)
#     >>> print(f'计算1-10的平方分别为：{res}')
#
#     """
#     # 定义n个线程池
#     pool = Pool(thread_num)
#     # 利用map让线程池中的所有线程‘同时’执行func函数
#     result = pool.map(func, param_list)
#     if is_ret:
#         return result


from multiprocessing import Pool, cpu_count
import time


def func1(x1):
    time.sleep(0.01)
    return (x1, x1 * x1)


if __name__ == '__main__':
    print('CPU核的数量：', cpu_count())
    begin = time.time()

    with Pool(5) as p:
        result = p.imap(func1, range(1000), chunksize=1)
        print(result)
        p.close()
        p.join()

    during = time.time() - begin

    print(during)
    # print([result.next() for r in range(100)])



import sys

from pymongo import MongoClient
from json import JSONEncoder
from bson import ObjectId

from archer_nlp.common_utils import except_info


class ObjectIdJSONEncoder(JSONEncoder):
    def default(self, o):
        if isinstance(o, ObjectId):
            return str(o)
        return JSONEncoder.default(self, o)


class Mongo:
    def __init__(self, config=None):
        self.config = config
        self.db = self.get_database()

    def get_database(self):
        try:
            client = MongoClient(self.config.get('host', '127.0.0.1'), self.config.get('port', 27017))
            db = client.get_database(self.config.get('db_name', ''))
            user = self.config.get("db_user", '')
            pwd = self.config.get("db_pwd", '')
            if user and pwd:
                db.authenticate(user, pwd)
            return db
        except Exception as e:
            print(except_info(e))
            raise Exception('connect to mongo error!')

    def get_list(self, table, where={}, columns=None):
        try:
            if columns is None:
                data = self.db[table].find(where)
            else:
                data = self.db[table].find(where, columns)

            return data
        except Exception as e:
            print(except_info(e))
            return None

    def get_list_by_limit(self, table, where={}, start=0, limit=30):
        try:
            return self.db[table].find(where).skip(start).limit(limit)
        except Exception as e:
            print(except_info(e))
            return None

    def get_count(self, table, where={}):
        try:
            if where:
                count = self.db[table].count_documents(where)
            else:
                count = self.db[table].estimated_document_count()
            return count
        except Exception as e:
            print(except_info(e))
            return None

    def insert(self, table, data):
        try:
            return self.db[table].insert(data)
        except Exception as e:
            print(except_info(e))
            return None

    def del_primary_id(self, data, where):
        """
        当更新操作的时候，由于data包含了_id，而where条件搜索出来的数据的主键不是该_id，会引起MongoDB异常
        :param data: 需要更新的数据
        :param where: 更新条件
        :return: 返回主键_id，注意有可能为None值
        """
        primary_id = None
        if where and data and '_id' in data and data['_id']:
            primary_id = data['_id']
            del data['_id']
        return primary_id

    def update(self, table, data, where={}, multi=False):
        try:
            primary_id = self.del_primary_id(data, where)
            return self.db[table].update(where, data, multi=multi)
        except Exception as e:
            print(except_info(e))
            return None
        finally:
            if primary_id:
                data['_id'] = primary_id

    def deleteMany(self, table, where={}):
        """
        清空数据，也可删除指定条件数据
        :param table:
        :param where:
        :return:
        """
        try:
            return self.db[table].delete_many(where)
        except Exception as e:
            print(except_info(e))
            return None



from urllib import request
from bs4 import BeautifulSoup
import requests

# content_type = 'application/json;charset=UTF-8'
content_type = 'application/json'
user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36 Edg/110.0.1587.57'

headers = [('Content-Type', content_type), ('User-Agent', user_agent)]
opener = request.build_opener()
opener.addheaders = headers
request.install_opener(opener)

requests_headers = {'Content-Type': content_type, 'User-Agent': user_agent}


# 常见流程
# 1、初始化bs：init_bs
# 2、用class name查找元素：find_element、find_elements、select_elements
# 3、获取元素内容：get_text、get_text_tag


def get_content(url):
    """
    获取页面内容
    :param url:
    :return:
    """
    # 如果慢的话，关闭代理。
    try:
        return request.urlopen(url).read().decode('UTF-8')
    except:
        # 上面的方法报错时调用
        return requests.get(url, headers=requests_headers).text


def init_bs(url):
    """
    初始化bs
    :param url:
    :return:
    """
    con = get_content(url)
    return BeautifulSoup(con, 'lxml')


def find_element(element, class_obj, class_name=''):
    """
    获取元素
    :param element:
    :param class_obj:
    :param class_name:
    :return:
    """
    return element.find(class_obj, attrs={'class': class_name})


def find_elements(element, class_obj, class_name=''):
    """
    获取多个元素
    :param element:
    :param class_obj:
    :param class_name:
    :return:
    """
    return element.find_all(class_obj, attrs={'class': class_name})


def select_elements(element, xpath_str):
    """
    使用xpath路径选择元素
    :param element:
    :param xpath_str:
    :return:
    """
    # element.select('div.short-field-item > div > p.short-field-title')
    return element.select(xpath_str)


def get_text(element):
    """
    获取文本内容（无HTML标签）
    :param element:
    :return:
    """
    return element.get_text().strip()


def get_text_tag(element):
    """
    获取带HTML标签（如<p>标签）的文本内容
    :param element:
    :return:
    """
    # element = find_element(soup, 'div', 'docYes')
    # get_text_tag(element).strip()
    return ''.join([str(ele) for ele in element.contents])
